<?php

if(!defined('PLX_ROOT')) { exit; }

/*
 * Add a markdown editor to PluXml for editing content of article
 * with spell checker
 * use medias.php for selecting a picture
 *
 * Copyright Jean-Pierre Pourrez ( aka bazooka07 : https://github.com/bazooka07/ )
 *
 * https://github.com/Ionaru/easy-markdown-editor
 * https://github.com/sparksuite/codemirror-spell-checker
 * https://github.com/pete-wildberger/codemirror-spell-checker
 * https://github.com/inkdropapp/codemirror-spell-checker
 *
 * availables dictionaries at : https://github.com/titoBouzout/Dictionaries
 * Don't forget to rename correctly theirs (E.G.: en_US.{aff,dic}  if stored in your "easy-markdown-editor/dist" folder
 *
 * https://github.com/erusev/parsedown
 *
 * 2020-02-25 : first release
 * */

class kzMarkdown extends plxPlugin {

	const DIC_URL_BASE = 'https://raw.githubusercontent.com/titoBouzout/Dictionaries/master/';
	const LIB_PATH = PLX_PLUGINS . __CLASS__ . '/easy-markdown-editor/dist/';

	public $html = true;

	public function __construct($default_lang) {
		parent::__construct($default_lang);

		if(!defined('PLX_ADMIN')) { return; }

		// parent::setConfigProfil(PROFIL_ADMIN);
		foreach(array(
			'AdminArticlePreview',
			'plxAdminDelArticle',
			'plxAdminEditArticle',
			'plxAdminEditArticleEnd',
			// 'AdminArticlePostData',
			'AdminArticleParseData',
			'AdminArticleInitData',
			'AdminArticleContent',
			'AdminArticleFoot',
			'AdminMediasFoot' // Required for using medias manager (core/admin/medias.php)
		) as $hook) {
			$this->addHook($hook, $hook);
		}

		if(filter_has_var(INPUT_POST, __CLASS__)) {
			include __DIR__ . '/parsedown/Parsedown.php';
		}
	}

	public function OnActivate() {
		$path = __DIR__ . '/css';
		if(is_dir($path) or mkdir($path)) {
			$content = file_get_contents(self::LIB_PATH . 'easymde.css');
			$content .= file_get_contents(__DIR__ . '/style.css');
			file_put_contents($path . '/admin.css', $content);
		} else {
			plxMsg::Error("$path is unwritable");
		}
	}

	public function OnDeActivate() {
		$filename = __DIR__ . '/css/admin.css';
		if(file_exists($filename)) {
			unlink($filename);
		}
	}

	public function AdminArticlePreview() {
		if(
			! filter_has_var(INPUT_POST, __CLASS__) or
			! filter_has_var(INPUT_POST, 'content')
		) {
			return;
		}

		echo <<< 'CODE'
<?php
$Parsedown = new Parsedown();
$art['content'] = $Parsedown->text(trim(filter_input(INPUT_POST, 'content', FILTER_SANITIZE_STRING)));
?>
CODE;
		echo PHP_EOL;
	}

	public function plxAdminDelArticle() {
		echo <<< 'CODE'
<?php
$filenames = glob(PLX_ROOT . $this->aConf['racine_articles'] . ".$id.*.md");
if(is_array($filenames) and count($filenames) > 0) {
	array_map(
		function($item) { unlink($item); },
		$filenames
	);
}
?>
CODE;
		echo PHP_EOL;
	}

	public function plxAdminEditArticle() {
		if(! filter_has_var(INPUT_POST, __CLASS__)) { return; }

		$this->saving = true;
		echo <<< 'CODE'
<?php
$kzContent = trim($content['content']);
$Parsedown = new Parsedown();
$content['content'] = $Parsedown->text($kzContent);
?>
CODE;
		echo PHP_EOL;
	}

	public function plxAdminEditArticleEnd() {
		if(empty($this->saving)) { return; }

		echo <<< 'CODE'
<?php
$filenames = glob(PLX_ROOT . $this->aConf['racine_articles'] . ".$id*.md");
if(is_array($filenames) and count($filenames) > 0) {
	array_map(
		function($item) { unlink($item); },
		$filenames
	);
}
plxUtils::write($kzContent, preg_replace('#([^/]*)\.xml$#', '.$1.md', $filename));
?>
CODE;
		echo PHP_EOL;
	}

	public function AdminArticleParseData() {
		$code = <<< 'CODE'
<?php
$filenames = glob(PLX_ROOT . $plxAdmin->aConf['racine_articles'] . ".$artId.*.md");
if(is_array($filenames) and count($filenames) > 0) {
	$content = file_get_contents($filenames[0]);
	$plxAdmin->plxPlugins->aPlugins['##CLASS##']->html = false;
}
?>
CODE;
		echo str_replace('##CLASS##', __CLASS__ , $code) . PHP_EOL;
	}

	public function AdminArticleInitData() {
		$this->html = false;
	}

	public function AdminArticleContent() {
		// Clean up all old tokens
		if(!empty($_SESSION[__CLASS__]) and is_array($_SESSION[__CLASS__])) {
			$now = time();
			foreach($_SESSION[__CLASS__] as $k=>$dead_time) {
				if($dead_time < $now) {
					unset($_SESSION[__CLASS__][$k]);
				}
			}
		}

		// Mark $_POST as markdown
		if(empty($this->html)) {
			$this->token = plxUtils::charAleatoire(32);
			$_SESSION[__CLASS__][$this->token] = time() + 3600; // seconds
?>
		<input type="hidden" id="id_<?= __CLASS__ ?>" name="<?= __CLASS__ ?>" value="<?= $this->token ?>" />
<?php
		}
	}

	private function _insertTextTable($columnsCount) {
		$title = $this->getLang('COLUMN');
		$cell = $this->getLang('TEXT');
		// $l1 = strlen($title);
		// if($l1 < strlen($cell)) { $l1 = strlen($cell); }
		$l1 = 3;
		$headers = array();
		for($i=1; $i<= $columnsCount; $i++) {
			$headers[] = str_pad("$title-$i", $l1);
		}
		$content = array(
			'[\'\', \'',
			'',
			'| ' . implode(' | ', $headers) . ' |',
			'|' . str_repeat(' ' . str_repeat('-', ($l1)) . ' |', $columnsCount),
			'|' . str_repeat(' - |', $columnsCount),
			'',
			'\']'
		);
		return implode('\n', $content);
	}

	public function AdminArticleFoot() {
		if(!empty($this->html)) { return; }

		$src = self::LIB_PATH .'easymde.min.js';
		$spellCheckUrl = self::DIC_URL_BASE . urlencode($this->getLang('DICT'));
		$lang = $this->getLang('LANG');
?>
		<div id="medias-container">
			<div>
				<button id="kzMarkdown-close">X</button>
				<iframe src="medias.php" id="kzMarkdown-explorer">Your browser doesn't support any iframe'</iframe>
			</div>
		</div>
		<script type="text/javascript" src="<?= $src ?>"></script>
		<script type="text/javascript">
			(function(id) {
				'use strict';

				const active = '<?= __CLASS__ ?>';
				var url = null;

				function updateUrls(content) {
					return content.replace(
						/!\[([^\]]*)\]\(([^)]*)\)/g,
						function(match, p1, p2) {
							if(p2.startsWith('http') || p2.startsWith('data:')) {
								return match;
							}
							return '![' + p1 + '](' + '<?= PLX_ROOT ?>' + p2 + ')';
						}
					)
				}

				// For preview : Markdown is rendered with Javascript
				function myPreviewRender1(aText) {
					return EasyMDE.prototype.markdown(updateUrls(aText));
				}

				// For preview : Markdown is rendered with PHP from the server
				function myPreviewRender2(aText, preview) {
					const xhr = new XMLHttpRequest();
					xhr.onload = function() {
						this.preview.innerHTML = decodeURI(this.responseText);
					};
					xhr.onerror = function() {
						console.error(this.status, this.statusText);
					}

					xhr.preview = preview;

					const fd = new FormData();
					fd.append('<?= __CLASS__ ?>-token', document.getElementById('id_<?= __CLASS__ ?>').value);
					fd.append('<?= __CLASS__ ?>-content', updateUrls(aText));

					xhr.open('POST', '<?= PLX_PLUGINS . __CLASS__ ?>/renderer.php');
					xhr.send(fd);

					return 'Loading..';
				}

				var easyMDE = null;
				const content = document.getElementById(id);
				if(content != null) {
					easyMDE = new EasyMDE({
						element: content,
						showIcons: ['strikethrough', 'code', 'table', 'redo', 'heading-1', 'heading-2', 'heading-3', 'undo', 'clean-block', 'horizontal-rule'],
						hideIcons: ['heading'],
						// hideIcons: ['heading', 'image'],
						autofocus: true,
						promptURLs:  true,
						// autoDownloadFontAwesome: false,
						// spellChecker : false,
						spellCheckUrl: '<?= $spellCheckUrl ?>',
						lang: '<?= $lang ?>',
						insertTexts: {
							table: <?= $this->_insertTextTable(6) . PHP_EOL ?>
						},
						previewRender: myPreviewRender2,
						imageCallback: function() {
							if(typeof url != 'string' || url.length == 0) {
								document.body.classList.add(active);
								return false;
							}

							const value = url;
							url = null;
							return value;
						}
					});
				} else {
					console.error('#id_content element not found');
				}

				document.getElementById('kzMarkdown-close').addEventListener(
					'click',
					function(event) {
						event.preventDefault();
						document.body.classList.remove(active);
					}
				);

				window.addEventListener('message', function(event) {
					event.preventDefault();
					if(event.origin == window.location.origin && event.data.startsWith('url:')) {
						document.body.classList.remove(active);
						url = event.data.replace(/^url:\s*/, '');
						if(url.length > 0) {
							easyMDE.drawImage();
						}
					}
				}, false);
			})('id_content');
		</script>
<?php
	}

	public function AdminMediasFoot() {
		// for medias.php in an <iframe>
?>
		<script type="text/javascript">
			(function() {
				if(window.frameElement != null && window.frameElement.id == 'kzMarkdown-explorer') {
					document.body.classList.add('mediasManager');
					document.body.classList.add('kzMarkdown');
					const ROOT = window.location.href.replace(/\bcore\/admin\/medias\.php$/, '');
					const tbody = document.querySelector('#medias-table tbody');
					if(tbody != null) {
						tbody.addEventListener('click',  function(event) {
							if(event.target.tagName == 'A') {
								event.preventDefault();
								var href = event.target.href;
								if(href.indexOf(ROOT) == 0) {
									href = href.substr(ROOT.length);
								}
								window.parent.postMessage('url:' + href, '*');
							}
						});
					}
				} else {
					console.error('No kzMarkdown-explorer');
				}
			})();
		</script>
<?php
	}
}
?>
